import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.csscms_data_model_models_subject_alternative_name import CSSCMSDataModelModelsSubjectAlternativeName


T = TypeVar("T", bound="CSSCMSDataModelModelsSSLEndpointHistoryResponseCertificateModel")


@_attrs_define
class CSSCMSDataModelModelsSSLEndpointHistoryResponseCertificateModel:
    """
    Attributes:
        id (Union[Unset, int]):
        issued_dn (Union[Unset, None, str]):
        serial_number (Union[Unset, None, str]):
        not_before (Union[Unset, datetime.datetime]):
        not_after (Union[Unset, datetime.datetime]):
        signing_algorithm (Union[Unset, None, str]):
        thumbprint (Union[Unset, None, str]):
        issuer_dn (Union[Unset, None, str]):
        issued_cn (Union[Unset, None, str]):
        subject_alt_name_elements (Union[Unset, None, List['CSSCMSDataModelModelsSubjectAlternativeName']]):
    """

    id: Union[Unset, int] = UNSET
    issued_dn: Union[Unset, None, str] = UNSET
    serial_number: Union[Unset, None, str] = UNSET
    not_before: Union[Unset, datetime.datetime] = UNSET
    not_after: Union[Unset, datetime.datetime] = UNSET
    signing_algorithm: Union[Unset, None, str] = UNSET
    thumbprint: Union[Unset, None, str] = UNSET
    issuer_dn: Union[Unset, None, str] = UNSET
    issued_cn: Union[Unset, None, str] = UNSET
    subject_alt_name_elements: Union[Unset, None, List["CSSCMSDataModelModelsSubjectAlternativeName"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        issued_dn = self.issued_dn
        serial_number = self.serial_number
        not_before: Union[Unset, str] = UNSET
        if not isinstance(self.not_before, Unset):
            not_before = self.not_before.isoformat()[:-6]+'Z'

        not_after: Union[Unset, str] = UNSET
        if not isinstance(self.not_after, Unset):
            not_after = self.not_after.isoformat()[:-6]+'Z'

        signing_algorithm = self.signing_algorithm
        thumbprint = self.thumbprint
        issuer_dn = self.issuer_dn
        issued_cn = self.issued_cn
        subject_alt_name_elements: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.subject_alt_name_elements, Unset):
            if self.subject_alt_name_elements is None:
                subject_alt_name_elements = None
            else:
                subject_alt_name_elements = []
                for subject_alt_name_elements_item_data in self.subject_alt_name_elements:
                    subject_alt_name_elements_item = subject_alt_name_elements_item_data.to_dict()

                    subject_alt_name_elements.append(subject_alt_name_elements_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if issued_dn is not UNSET:
            field_dict["issuedDN"] = issued_dn
        if serial_number is not UNSET:
            field_dict["serialNumber"] = serial_number
        if not_before is not UNSET:
            field_dict["notBefore"] = not_before
        if not_after is not UNSET:
            field_dict["notAfter"] = not_after
        if signing_algorithm is not UNSET:
            field_dict["signingAlgorithm"] = signing_algorithm
        if thumbprint is not UNSET:
            field_dict["thumbprint"] = thumbprint
        if issuer_dn is not UNSET:
            field_dict["issuerDN"] = issuer_dn
        if issued_cn is not UNSET:
            field_dict["issuedCN"] = issued_cn
        if subject_alt_name_elements is not UNSET:
            field_dict["subjectAltNameElements"] = subject_alt_name_elements

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.csscms_data_model_models_subject_alternative_name import (
            CSSCMSDataModelModelsSubjectAlternativeName,
        )

        d = {x[0].lower()+x[1:]:src_dict[x] for x in src_dict.keys()}
        id = d.pop("id", UNSET)

        issued_dn = d.pop("issuedDN", UNSET)

        serial_number = d.pop("serialNumber", UNSET)

        _not_before = d.pop("notBefore", UNSET)
        not_before: Union[Unset, datetime.datetime]
        if isinstance(_not_before, Unset):
            not_before = UNSET
        else:
            not_before = isoparse(_not_before)

        _not_after = d.pop("notAfter", UNSET)
        not_after: Union[Unset, datetime.datetime]
        if isinstance(_not_after, Unset):
            not_after = UNSET
        else:
            not_after = isoparse(_not_after)

        signing_algorithm = d.pop("signingAlgorithm", UNSET)

        thumbprint = d.pop("thumbprint", UNSET)

        issuer_dn = d.pop("issuerDN", UNSET)

        issued_cn = d.pop("issuedCN", UNSET)

        subject_alt_name_elements = []
        _subject_alt_name_elements = d.pop("subjectAltNameElements", UNSET)
        for subject_alt_name_elements_item_data in _subject_alt_name_elements or []:
            subject_alt_name_elements_item = CSSCMSDataModelModelsSubjectAlternativeName.from_dict(
                subject_alt_name_elements_item_data
            )

            subject_alt_name_elements.append(subject_alt_name_elements_item)

        csscms_data_model_models_ssl_endpoint_history_response_certificate_model = cls(
            id=id,
            issued_dn=issued_dn,
            serial_number=serial_number,
            not_before=not_before,
            not_after=not_after,
            signing_algorithm=signing_algorithm,
            thumbprint=thumbprint,
            issuer_dn=issuer_dn,
            issued_cn=issued_cn,
            subject_alt_name_elements=subject_alt_name_elements,
        )

        return csscms_data_model_models_ssl_endpoint_history_response_certificate_model
